@@ -6,9 +6,11 @@ import re
 import sys
 import xml.parsers.expat
 
-__version__ = "0.2.0"
+__version__ = "0.2.2"
 
+DEFAULT_BLANKS = False
 DEFAULT_COMPRESS = False
+DEFAULT_SELFCLOSE = False
 DEFAULT_CORRECT = True
 DEFAULT_INDENT = 2
 DEFAULT_INDENT_CHAR = " "
@@ -25,7 +27,9 @@ class Formatter:
         self,
         indent=DEFAULT_INDENT,
         preserve=[],
+        blanks=DEFAULT_BLANKS,
         compress=DEFAULT_COMPRESS,
+        selfclose=DEFAULT_SELFCLOSE,
         indent_char=DEFAULT_INDENT_CHAR,
         encoding_input=DEFAULT_ENCODING_INPUT,
         encoding_output=DEFAULT_ENCODING_OUTPUT,
@@ -34,6 +38,8 @@ class Formatter:
     ):
         # Minify the XML document:
         self.compress = compress
+        # Use self-closing tags
+        self.selfclose = selfclose
         # Correct text nodes
         self.correct = correct
         # Decode the XML document:
@@ -48,6 +54,8 @@ class Formatter:
         self.inline = inline
         # Don't compress this elements and their descendants:
         self.preserve = preserve
+        # Preserve blanks lines (collapse multiple into one)
+        self.blanks = blanks
 
     @property
     def encoding_effective(self, enc=None):
@@ -523,7 +531,10 @@ class Formatter:
             if not self.preserve and not self.cdata_section:
                 # remove empty tokens always in element content!
                 if self.empty and not self.descendant_mixed:
-                    str = ""
+                    if self.formatter.blanks and not self.formatter.compress and re.match(r"\s*\n\s*\n\s*", str):
+                        str = "\n"
+                    else:
+                        str = ""
                 else:
                     if self.correct:
                         str = re.sub(r"\r\n", "\n", str)
@@ -629,7 +640,7 @@ class Formatter:
             str = ""
             # Don't close empty nodes on compression mode:
             if (
-                not self.formatter.compress
+                not (self.formatter.compress or self.formatter.selfclose)
                 or self.list[self.pos - 1].name != "StartElement"
             ):
                 if self.preserve in [0] and self.indent:
@@ -707,7 +718,7 @@ class Formatter:
             str += "<%s" % self.arg[0]
             for attr in sorted(self.arg[1].keys()):
                 str += self.attribute(attr, self.arg[1][attr])
-            if self.list[self.pos + 1].end and self.formatter.compress:
+            if self.list[self.pos + 1].end and (self.formatter.compress or self.formatter.selfclose):
                 str += "/>"
             else:
                 str += ">"
@@ -740,10 +751,11 @@ def cli_usage(msg=""):
     """ Output usage for command line tool. """
     sys.stderr.write(msg + "\n")
     sys.stderr.write(
-        'Usage: xmlformat [--preserve "pre,literal"]\
- [--compress] [--indent num] [--indent-char char] [--outfile file]\
- [--encoding enc] [--outencoding enc] [--disable-inlineformatting]\
- [--overwrite] [--disable-correction] [--help] <--infile file | file | - >\n'
+        'Usage: xmlformat [--preserve "pre,literal"] [--blanks]\
+ [--compress] [--selfclose] [--indent num] [--indent-char char]\
+ [--outfile file] [--encoding enc] [--outencoding enc]\
+ [--disable-inlineformatting] [--overwrite] [--disable-correction]\
+ [--help] <--infile file | file | - >\n'
     )
     sys.exit(2)
 
@@ -756,7 +768,9 @@ def cli():
     outfile = None
     overwrite = False
     preserve = []
+    blanks = False
     compress = DEFAULT_COMPRESS
+    selfclose = DEFAULT_SELFCLOSE
     infile = None
     encoding = DEFAULT_ENCODING_INPUT
     outencoding = DEFAULT_ENCODING_OUTPUT
@@ -768,6 +782,7 @@ def cli():
             "",
             [
                 "compress",
+                "selfclose",
                 "disable-correction",
                 "disable-inlineformatting",
                 "encoding=",
@@ -779,6 +794,7 @@ def cli():
                 "outencoding=",
                 "overwrite",
                 "preserve=",
+                "blanks",
             ],
         )
     except getopt.GetoptError as err:
@@ -788,10 +804,14 @@ def cli():
             indent = value
         elif key in ["--preserve"]:
             preserve = value.replace(",", " ").split()
+        elif key in ["--blanks"]:
+            blanks = True
         elif key in ["--help"]:
             cli_usage()
         elif key in ["--compress"]:
             compress = True
+        elif key in ["--selfclose"]:
+            selfclose = True
         elif key in ["--outfile"]:
             outfile = value
         elif key in ["--infile"]:
@@ -812,7 +832,9 @@ def cli():
         formatter = Formatter(
             indent=indent,
             preserve=preserve,
+            blanks=blanks,
             compress=compress,
+            selfclose=selfclose,
             encoding_input=encoding,
             encoding_output=outencoding,
             indent_char=indent_char,
